"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const index_1 = require("../utils/index");
const test_1 = require("@schematics/angular/utility/test");
const index_2 = require("../utils/new-app/index");
function default_1(options) {
    if (!options.directory) {
        options.directory = options.name;
    }
    const templateSource = schematics_1.apply(schematics_1.url('./files'), [
        schematics_1.move(options.directory),
    ]);
    let rule = schematics_1.chain([
        schematics_1.externalSchematic('@schematics/angular', 'ng-new', options),
        adjustCLIConfig(options),
        index_2.newApp(options),
        schematics_1.mergeWith(templateSource, schematics_1.MergeStrategy.Overwrite),
        updatePackageJson(options)
    ]);
    return rule;
}
exports.default = default_1;
function updatePackageJson(options) {
    return schematics_1.chain([
        index_1.addDependencyToPackageJson(options, '@angular/service-worker', '^6.0.0-rc.6'),
        index_1.addDependencyToPackageJson(options, '@angular/platform-server', '^6.0.0-rc.6'),
        index_1.addDependencyToPackageJson(options, '@angular/cdk', '^6.0.0-rc.6'),
        index_1.addDependencyToPackageJson(options, '@angular/material', '^6.0.0-rc.6'),
        index_1.addDependencyToPackageJson(options, 'webpack-cli', '2.1.2'),
        index_1.addDependencyToPackageJson(options, 'ts-loader', '4.2.0', true),
        index_1.addOrReplaceScriptInPackageJson(options, 'build:client-and-server-bundles', 'ng build --prod && ng run __projectName__:server'),
        index_1.addOrReplaceScriptInPackageJson(options, 'build:prod', 'npm run build:client-and-server-bundles && npm run webpack:server'),
        index_1.addDependencyToPackageJson(options, '@ngx-translate/core', '^10.0.1'),
        index_1.addDependencyToPackageJson(options, '@ngx-translate/http-loader', '^3.0.1'),
        tree => {
            let packageJsonContent = test_1.getFileContent(tree, `${options.directory}/package.json`);
            packageJsonContent = packageJsonContent.replace('__projectName__', options.name);
            index_1.createOrOverwriteFile(tree, `${options.directory}/package.json`, packageJsonContent);
            return tree;
        }
    ]);
}
function adjustCLIConfig(options) {
    return tree => {
        const cliConfig = JSON.parse(test_1.getFileContent(tree, `${options.directory}/angular.json`));
        // delete cliConfig.projects[options.name].sourceRoot;
        cliConfig.projects[options.name].architect.build.options.outputPath = 'dist/browser';
        cliConfig.projects[options.name].architect.build.options.main = 'src/main.browser.ts';
        cliConfig.projects[options.name].architect.build.options.assets.push({ glob: "manifest.json", input: "src", output: "/" });
        cliConfig.projects[options.name].architect.build.options.assets.push({ glob: "ngsw-worker.js", input: "src/assets/fakeSW", output: "/" });
        cliConfig.projects[options.name].architect.build.options.styles = [{ input: "src/styles/main.scss" }];
        cliConfig.projects[options.name].architect.build.configurations.production.serviceWorker = true;
        delete cliConfig.defaultProject;
        cliConfig.projects[options.name].architect.serve.configurations.dev = { browserTarget: `${options.name}:build:dev` };
        delete cliConfig.projects[options.name].architect.serve.configurations.production;
        cliConfig.projects[options.name].architect.test.options.assets.push({ glob: "manifest.json", input: "src", output: "/" });
        cliConfig.projects[options.name].architect.test.options.styles = [{ input: "src/styles/main.scss" }];
        cliConfig.projects[options.name].architect.server = {
            builder: '@angular-devkit/build-angular:server',
            options: {
                outputPath: 'dist/server',
                main: 'src/main.server.ts',
                tsConfig: 'src/tsconfig.server.json'
            }
        };
        cliConfig.projects[`${options.name}-e2e`].architect.e2e.options.devServerTarget = `${options.name}:serve`;
        index_1.createOrOverwriteFile(tree, `${options.directory}/angular.json`, JSON.stringify(cliConfig, null, "  "));
        return tree;
    };
}
//# sourceMappingURL=index.js.map